"""Premises Extender Mixin"""

import sys
import importlib


class ExtenderMixin():
    """Dynamic Premises processing"""

    @property
    def rule(self):
        """Returns premises rule class"""
        rule_num = ''.join(['0' if self.is_empty(k) else '1' for k in self.building_attrs])
        package_name = getattr(sys.modules[__name__], '__package__', None)
        module_name = f'rule{rule_num}'
        module = importlib.import_module(f'{package_name}.{module_name}')
        class_name = f'Rule{rule_num}'
        return getattr(module, class_name)

    def extend(self):
        """Dynamically extends instance with appropriate rule"""
        base_cls = self.__class__
        object.__setattr__(self, '__class__', type(base_cls.__name__, (base_cls, self.rule), {}))



"""Rule 6"""

from .premisable import PremisableMixin


class Rule110(PremisableMixin):
    """Rule 6 processing"""

    @property
    def rule_attrs(self):
        """Returns premises list"""
        if self.is_exception('sub_building_name'):
            if self.is_split_exception('building_name'):
                return (
                    'sub_and_building_name_but_last_word',
                    'name_last_word_and_thoroughfare_or_locality'
                    )
            return ('sub_name_and_name',)
        if self.is_exception('building_name'):
            return ('sub_building_name', 'name_and_thoroughfare_or_locality')
        if self.is_split_exception('building_name'):
            return (
                'sub_building_name', 'building_name_but_last_word',
                'name_last_word_and_thoroughfare_or_locality'
                )
        return ('sub_building_name', 'building_name')

    @property
    def includes_first_thoroughfare_or_locality(self):
        """Returns if premises includes first thoroughfare or locality"""
        return (
            self.is_split_exception('building_name')
            or (not self.is_exception('sub_building_name') and self.is_exception('building_name'))
            )
